"""
AnySpecs CLI - Code is Cheap, Show me Any Specs

A unified CLI tool for exporting chat history from multiple AI assistants.
Supports Cursor AI and Claude Code with various export formats.
"""

__version__ = "0.0.5"
__author__ = "AnySpecs Team"

from .cli import main

__all__ = ["main"] 